@@ -1,10 +1,10 @@
 ;;; ob-latex.el --- Babel Functions for LaTeX        -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2009-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2009-2026 Free Software Foundation, Inc.
 
 ;; Author: Eric Schulte
 ;; Keywords: literate programming, reproducible research
-;; Homepage: https://orgmode.org
+;; URL: https://orgmode.org
 
 ;; This file is part of GNU Emacs.
 
@@ -30,6 +30,10 @@
 ;; be created directly form the latex source code.
 
 ;;; Code:
+
+(require 'org-macs)
+(org-assert-version)
+
 (require 'ob)
 (require 'org-macs)
 
@@ -45,6 +49,7 @@
 (defvar org-format-latex-options)	  ; From org.el
 (defvar org-latex-default-packages-alist) ; From org.el
 (defvar org-latex-packages-alist)	  ; From org.el
+(defvar org-preview-latex-process-alist)  ; From org.el
 
 (defvar org-babel-default-header-args:latex
   '((:results . "latex") (:exports . "results"))
@@ -61,21 +66,92 @@
     (pdfpng       . :any)
     (pdfwidth     . :any)
     (headers      . :any)
-    (packages     . :any)
     (buffer       . ((yes no))))
   "LaTeX-specific header arguments.")
 
 (defcustom org-babel-latex-htlatex "htlatex"
-  "The htlatex command to enable conversion of latex to SVG or HTML."
+  "The htlatex command to enable conversion of LaTeX to SVG or HTML."
   :group 'org-babel
   :type 'string)
 
+(defcustom org-babel-latex-preamble
+  (lambda (_)
+    "\\documentclass[preview]{standalone}
+")
+  "Closure which evaluates at runtime to the LaTeX preamble.
+
+It takes 1 argument which is the parameters of the source block."
+  :group 'org-babel
+  :package-version '(Org . "9.8")
+  :type 'function
+  :safe nil)
+
+(defcustom org-babel-latex-begin-env
+  (lambda (_)
+    "\\begin{document}")
+  "Function that evaluates to the begin part of the document environment.
+
+It takes 1 argument which is the parameters of the source block.
+This allows adding additional code that will be ignored when
+exporting the literal LaTeX source."
+  :group 'org-babel
+  :type 'function)
+
+(defcustom org-babel-latex-end-env
+  (lambda (_)
+    "\\end{document}")
+  "Closure which evaluates at runtime to the end part of the document environment.
+
+It takes 1 argument which is the parameters of the source block.
+This allows adding additional code that will be ignored when
+exporting the literal LaTeX source."
+  :group 'org-babel
+  :type 'function)
+
+(defcustom org-babel-latex-pdf-svg-process
+  "inkscape \
+--pdf-poppler \
+--export-area-drawing \
+--export-text-to-path \
+--export-plain-svg \
+--export-filename=%O \
+%f"
+  "Command to convert a PDF file to an SVG file."
+  :group 'org-babel
+  :type 'string
+  :package-version '(Org . "9.6"))
+
 (defcustom org-babel-latex-htlatex-packages
   '("[usenames]{color}" "{tikz}" "{color}" "{listings}" "{amsmath}")
   "Packages to use for htlatex export."
   :group 'org-babel
   :type '(repeat (string)))
 
+(defcustom org-babel-latex-process-alist
+  `((png :programs ("latex" "dvipng") :description "dvi > png"
+	 :message
+	 "you need to install the programs: latex and dvipng."
+	 :image-input-type "dvi" :image-output-type "png"
+	 :image-size-adjust (1.0 . 1.0) :latex-compiler
+         ,(if (executable-find "latexmk")
+              '("latexmk -f -pdf -latex -interaction=nonstopmode -output-directory=%o %f")
+            '("latex -interaction nonstopmode -output-directory %o %f"
+              "latex -interaction nonstopmode -output-directory %o %f"
+              "latex -interaction nonstopmode -output-directory %o %f"))
+	 :image-converter ("dvipng -D %D -T tight -o %O %f")
+	 :transparent-image-converter
+	 ("dvipng -D %D -T tight -bg Transparent -o %O %f")))
+  "Definitions of external processes for LaTeX result generation.
+See `org-preview-latex-process-alist' for more details.
+
+The following process symbols are recognized:
+- `png' :: Process used to produce .png output."
+  :group 'org-babel
+  :package-version '(Org . "9.8")
+  :type '(alist :tag "LaTeX to image backends"
+		:value-type (plist))
+  :safe nil)
+
 (defun org-babel-expand-body:latex (body params)
   "Expand BODY according to PARAMS, return the expanded body."
   (mapc (lambda (pair) ;; replace variables
@@ -84,12 +160,18 @@
                  (regexp-quote (format "%S" (car pair)))
                  (if (stringp (cdr pair))
                      (cdr pair) (format "%S" (cdr pair)))
-                 body)))
+                 body t t)))
 	(org-babel--get-vars params))
-  (org-trim body))
+  (let ((prologue (cdr (assq :prologue params)))
+        (epilogue (cdr (assq :epilogue params))))
+    (org-trim
+     (concat
+      (and prologue (concat prologue "\n"))
+      body
+      (and epilogue (concat "\n" epilogue "\n"))))))
 
 (defun org-babel-execute:latex (body params)
-  "Execute a block of Latex code with Babel.
+  "Execute LaTeX BODY according to PARAMS.
 This function is called by `org-babel-execute-src-block'."
   (setq body (org-babel-expand-body:latex body params))
   (if (cdr (assq :file params))
@@ -111,15 +193,30 @@ This function is called by `org-babel-execute-src-block'."
          ((and (string-suffix-p ".png" out-file) (not imagemagick))
           (let ((org-format-latex-header
 		 (concat org-format-latex-header "\n"
-			 (mapconcat #'identity headers "\n"))))
-	   (org-create-formula-image
-            body out-file org-format-latex-options in-buffer)))
+			 (mapconcat #'identity headers "\n")))
+                (org-preview-latex-process-alist org-babel-latex-process-alist))
+	    (org-create-formula-image
+             body out-file org-format-latex-options in-buffer 'png)))
+	 ((string= "svg" extension)
+	  (with-temp-file tex-file
+	    (insert (concat (funcall org-babel-latex-preamble params)
+			    (mapconcat #'identity headers "\n")
+			    (funcall org-babel-latex-begin-env params)
+			    body
+			    (funcall org-babel-latex-end-env params))))
+	  (let ((tmp-pdf (org-babel-latex-tex-to-pdf tex-file)))
+            (let* ((log-buf (get-buffer-create "*Org Babel LaTeX Output*"))
+                   (err-msg "org babel latex failed")
+                   (img-out (org-compile-file
+	                     tmp-pdf
+                             (list org-babel-latex-pdf-svg-process)
+                             extension err-msg log-buf)))
+              (rename-file img-out out-file t))))
          ((string-suffix-p ".tikz" out-file)
 	  (when (file-exists-p out-file) (delete-file out-file))
 	  (with-temp-file out-file
 	    (insert body)))
-	 ((and (or (string= "svg" extension)
-		   (string= "html" extension))
+	 ((and (string= "html" extension)
 	       (executable-find org-babel-latex-htlatex))
 	  ;; TODO: this is a very different way of generating the
 	  ;; frame latex document than in the pdf case.  Ideally, both
@@ -152,20 +249,19 @@ This function is called by `org-babel-execute-src-block'."
 	    (if (string-suffix-p ".svg" out-file)
 		(progn
 		  (shell-command "pwd")
-		  (shell-command (format "mv %s %s"
-					 (concat (file-name-sans-extension tex-file) "-1.svg")
-					 out-file)))
+                  (rename-file (concat (file-name-sans-extension tex-file) "-1.svg")
+                               out-file t))
 	      (error "SVG file produced but HTML file requested")))
 	   ((file-exists-p (concat (file-name-sans-extension tex-file) ".html"))
 	    (if (string-suffix-p ".html" out-file)
-		(shell-command "mv %s %s"
-			       (concat (file-name-sans-extension tex-file)
-				       ".html")
-			       out-file)
-	      (error "HTML file produced but SVG file requested")))))
+                (rename-file (concat (file-name-sans-extension tex-file) ".html")
+                             out-file t)
+              (error "HTML file produced but SVG file requested")))))
 	 ((or (string= "pdf" extension) imagemagick)
 	  (with-temp-file tex-file
 	    (require 'ox-latex)
+            (defvar org-latex-compiler)
+            (declare-function org-latex--remove-packages "ox-latex" (pkg-alist info))
 	    (insert
 	     (org-latex-guess-inputenc
 	      (org-splice-latex-header
@@ -176,8 +272,12 @@ This function is called by `org-babel-execute-src-block'."
 		 (lambda (el)
 		   (unless (and (listp el) (string= "hyperref" (cadr el)))
 		     el))
-		 org-latex-default-packages-alist))
-	       org-latex-packages-alist
+		 (org-latex--remove-packages
+                  org-latex-default-packages-alist
+                  (list :latex-compiler org-latex-compiler))))
+               (org-latex--remove-packages
+	        org-latex-packages-alist
+                (list :latex-compiler org-latex-compiler))
 	       nil))
 	     (if fit "\n\\usepackage[active, tightpage]{preview}\n" "")
 	     (if border (format "\\setlength{\\PreviewBorder}{%s}" border) "")
@@ -210,7 +310,9 @@ This function is called by `org-babel-execute-src-block'."
     body))
 
 (defun org-babel-latex-convert-pdf (pdffile out-file im-in-options im-out-options)
-  "Generate a file from a pdf file using imagemagick."
+  "Generate OUT-FILE from PDFFILE using imagemagick.
+IM-IN-OPTIONS are command line options for input file, as a string;
+and IM-OUT-OPTIONS are the output file options."
   (let ((cmd (concat "convert " im-in-options " " pdffile " "
 		     im-out-options " " out-file)))
     (message "Converting pdffile file %s..." cmd)
