@@ -1,6 +1,6 @@
-;;; ol-eshell.el - Links to Working Directories in Eshell -*- lexical-binding: t; -*-
+;;; ol-eshell.el --- Links to Working Directories in Eshell  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2011-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2011-2026 Free Software Foundation, Inc.
 
 ;; Author: Konrad Hinsen <konrad.hinsen AT fastmail.net>
 
@@ -23,21 +23,24 @@
 
 ;;; Code:
 
+(require 'org-macs)
+(org-assert-version)
+
 (require 'eshell)
 (require 'esh-mode)
 (require 'ol)
 
-(declare-function eshell/pwd "em-dirs.el" (&rest args))
+(declare-function eshell/pwd "em-dirs" ())
 
 (org-link-set-parameters "eshell"
 			 :follow #'org-eshell-open
 			 :store #'org-eshell-store-link)
 
 (defun org-eshell-open (link _)
-  "Switch to an eshell buffer and execute a command line.
-   The link can be just a command line (executed in the default
-   eshell buffer) or a command line prefixed by a buffer name
-   followed by a colon."
+  "Switch to an eshell buffer and execute a command line for LINK.
+The LINK can be just a command line (executed in the default
+eshell buffer) or a command line prefixed by a buffer name
+followed by a colon."
   (let* ((buffer-and-command
           (if (string-match "\\([A-Za-z0-9+*-]+\\):\\(.*\\)" link)
 	      (list (match-string 1 link)
@@ -46,16 +49,21 @@
          (eshell-buffer-name (car buffer-and-command))
          (command (cadr buffer-and-command)))
     (if (get-buffer eshell-buffer-name)
-	(pop-to-buffer-same-window eshell-buffer-name)
+        (pop-to-buffer
+         eshell-buffer-name
+         (if (boundp 'display-comint-buffer-action) ; Emacs >= 29, <= 30
+             display-comint-buffer-action
+           '(display-buffer-same-window (inhibit-same-window) (category . comint))))
       (eshell))
     (goto-char (point-max))
     (eshell-kill-input)
     (insert command)
     (eshell-send-input)))
 
-(defun org-eshell-store-link ()
-  "Store a link that, when opened, switches back to the current eshell buffer
-   and the current working directory."
+(defun org-eshell-store-link (&optional _interactive?)
+  "Store eshell link.
+When opened, the link switches back to the current eshell buffer and
+the current working directory."
   (when (eq major-mode 'eshell-mode)
     (let* ((command (concat "cd " (eshell/pwd)))
            (link  (concat (buffer-name) ":" command)))
