@@ -1,9 +1,9 @@
 ;;; org-duration.el --- Library handling durations   -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2017-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2017-2026 Free Software Foundation, Inc.
 
 ;; Author: Nicolas Goaziou <mail@nicolasgoaziou.fr>
-;; Keywords: outlines, hypermedia, calendar, wp
+;; Keywords: outlines, hypermedia, calendar, text
 
 ;; This file is part of GNU Emacs.
 
@@ -51,6 +51,9 @@
 
 ;;; Code:
 
+(require 'org-macs)
+(org-assert-version)
+
 (require 'cl-lib)
 (require 'org-macs)
 
@@ -97,7 +100,11 @@ sure to call the following command:
   :group 'org-agenda
   :version "26.1"
   :package-version '(Org . "9.1")
-  :set (lambda (var val) (set-default var val) (org-duration-set-regexps))
+  :set (lambda (var val)
+         (set-default-toplevel-value var val)
+         ;; Avoid recursive load at startup.
+	 (when (featurep 'org-duration)
+           (org-duration-set-regexps)))
   :initialize 'custom-initialize-changed
   :type '(choice
 	  (const :tag "H:MM" h:mm)
@@ -280,30 +287,31 @@ translated into 0.0.
 
 Return value as a float.  Raise an error if duration format is
 not recognized."
-  (cond
-   ((equal duration "") 0.0)
-   ((numberp duration) (float duration))
-   ((string-match-p org-duration--h:mm-re duration)
-    (pcase-let ((`(,hours ,minutes ,seconds)
-		 (mapcar #'string-to-number (split-string duration ":"))))
-      (+ (/ (or seconds 0) 60.0) minutes (* 60 hours))))
-   ((string-match-p org-duration--full-re duration)
-    (let ((minutes 0)
-	  (s 0))
-      (while (string-match org-duration--unit-re duration s)
-	(setq s (match-end 0))
-	(let ((value (string-to-number (match-string 1 duration)))
-	      (unit (match-string 2 duration)))
-	  (cl-incf minutes (* value (org-duration--modifier unit canonical)))))
-      (float minutes)))
-   ((string-match org-duration--mixed-re duration)
-    (let ((units-part (match-string 1 duration))
-	  (hms-part (match-string 2 duration)))
-      (+ (org-duration-to-minutes units-part)
-	 (org-duration-to-minutes hms-part))))
-   ((string-match-p "\\`[0-9]+\\(\\.[0-9]*\\)?\\'" duration)
-    (float (string-to-number duration)))
-   (t (error "Invalid duration format: %S" duration))))
+  (save-match-data
+    (cond
+     ((equal duration "") 0.0)
+     ((numberp duration) (float duration))
+     ((string-match-p org-duration--h:mm-re duration)
+      (pcase-let ((`(,hours ,minutes ,seconds)
+		   (mapcar #'string-to-number (split-string duration ":"))))
+        (+ (/ (or seconds 0) 60.0) minutes (* 60 hours))))
+     ((string-match-p org-duration--full-re duration)
+      (let ((minutes 0)
+	    (s 0))
+        (while (string-match org-duration--unit-re duration s)
+	  (setq s (match-end 0))
+	  (let ((value (string-to-number (match-string 1 duration)))
+	        (unit (match-string 2 duration)))
+	    (cl-incf minutes (* value (org-duration--modifier unit canonical)))))
+        (float minutes)))
+     ((string-match org-duration--mixed-re duration)
+      (let ((units-part (match-string 1 duration))
+	    (hms-part (match-string 2 duration)))
+        (+ (org-duration-to-minutes units-part)
+	   (org-duration-to-minutes hms-part))))
+     ((string-match-p "\\`[0-9]+\\(\\.[0-9]*\\)?\\'" duration)
+      (float (string-to-number duration)))
+     (t (error "Invalid duration format: %S" duration)))))
 
 ;;;###autoload
 (defun org-duration-from-minutes (minutes &optional fmt canonical)
@@ -316,109 +324,110 @@ When optional argument CANONICAL is non-nil, ignore
 `org-duration-units' and use standard time units value.
 
 Raise an error if expected format is unknown."
-  (pcase (or fmt org-duration-format)
-    (`h:mm
-     (format "%d:%02d" (/ minutes 60) (mod minutes 60)))
-    (`h:mm:ss
-     (let* ((whole-minutes (floor minutes))
-	    (seconds (mod (* 60 minutes) 60)))
-       (format "%s:%02d"
-	       (org-duration-from-minutes whole-minutes 'h:mm)
-	       seconds)))
-    ((pred atom) (error "Invalid duration format specification: %S" fmt))
-    ;; Mixed format.  Call recursively the function on both parts.
-    ((and duration-format
-	  (let `(special . ,(and mode (or `h:mm:ss `h:mm)))
-	    (assq 'special duration-format)))
-     (let* ((truncated-format
-	     ;; Remove "special" mode from duration format in order to
-	     ;; recurse properly.  Also remove units smaller or equal
-	     ;; to an hour since H:MM part takes care of it.
-	     (cl-remove-if-not
-	      (lambda (pair)
-		(pcase pair
-		  (`(,(and unit (pred stringp)) . ,_)
-		   (> (org-duration--modifier unit canonical) 60))
-		  (_ nil)))
-	      duration-format))
-	    (min-modifier		;smallest modifier above hour
-	     (and truncated-format
-		  (apply #'min
-			 (mapcar (lambda (p)
-				   (org-duration--modifier (car p) canonical))
-				 truncated-format)))))
-       (if (or (null min-modifier) (< minutes min-modifier))
-	   ;; There is not unit above the hour or the smallest unit
-	   ;; above the hour is too large for the number of minutes we
-	   ;; need to represent.  Use H:MM or H:MM:SS syntax.
-	   (org-duration-from-minutes minutes mode canonical)
-	 ;; Represent minutes above hour using provided units and H:MM
-	 ;; or H:MM:SS below.
-	 (let* ((units-part (* min-modifier (/ (floor minutes) min-modifier)))
-		(minutes-part (- minutes units-part))
-		(compact (memq 'compact duration-format)))
-	   (concat
-	    (org-duration-from-minutes units-part truncated-format canonical)
-	    (and (not compact) " ")
-	    (org-duration-from-minutes minutes-part mode))))))
-    ;; Units format.
-    (duration-format
-     (let* ((fractional
-	     (let ((digits (cdr (assq 'special duration-format))))
-	       (and digits
-		    (or (wholenump digits)
-			(error "Unknown formatting directive: %S" digits))
-		    (format "%%.%df" digits))))
-	    (selected-units
-	     (sort (cl-remove-if
-		    ;; Ignore special format cells and compact option.
-		    (lambda (pair)
-		      (pcase pair
-			((or `compact `(special . ,_)) t)
-			(_ nil)))
-		    duration-format)
-		   (lambda (a b)
-		     (> (org-duration--modifier (car a) canonical)
-			(org-duration--modifier (car b) canonical)))))
-	    (separator (if (memq 'compact duration-format) "" " ")))
-       (cond
-	;; Fractional duration: use first unit that is either required
-	;; or smaller than MINUTES.
-	(fractional
-	 (let* ((unit (car
-		       (or (cl-find-if
-			    (lambda (pair)
-			      (pcase pair
-				(`(,u . ,req?)
-				 (or req?
-				     (<= (org-duration--modifier u canonical)
-					 minutes)))))
-			    selected-units)
-			   ;; Fall back to smallest unit.
-			   (org-last selected-units))))
-		(modifier (org-duration--modifier unit canonical)))
-	   (concat (format fractional (/ (float minutes) modifier)) unit)))
-	;; Otherwise build duration string according to available
-	;; units.
-	((org-string-nw-p
-	  (org-trim
-	   (mapconcat
-	    (lambda (units)
-	      (pcase-let* ((`(,unit . ,required?) units)
-			   (modifier (org-duration--modifier unit canonical)))
-		(cond ((<= modifier minutes)
-		       (let ((value (floor minutes modifier)))
-			 (cl-decf minutes (* value modifier))
-			 (format "%s%d%s" separator value unit)))
-		      (required? (concat separator "0" unit))
-		      (t ""))))
-	    selected-units
-	    ""))))
-	;; No unit can properly represent MINUTES.  Use the smallest
-	;; one anyway.
-	(t
-	 (pcase-let ((`((,unit . ,_)) (last selected-units)))
-	   (concat "0" unit))))))))
+  (if (< minutes 0) (concat "-" (org-duration-from-minutes (abs minutes) fmt canonical))
+    (pcase (or fmt org-duration-format)
+      (`h:mm
+       (format "%d:%02d" (/ minutes 60) (mod minutes 60)))
+      (`h:mm:ss
+       (let* ((whole-minutes (floor minutes))
+	      (seconds (mod (* 60 minutes) 60)))
+         (format "%s:%02d"
+	         (org-duration-from-minutes whole-minutes 'h:mm)
+	         seconds)))
+      ((pred atom) (error "Invalid duration format specification: %S" fmt))
+      ;; Mixed format.  Call recursively the function on both parts.
+      ((and duration-format
+	    (let `(special . ,(and mode (or `h:mm:ss `h:mm)))
+	      (assq 'special duration-format)))
+       (let* ((truncated-format
+	       ;; Remove "special" mode from duration format in order to
+	       ;; recurse properly.  Also remove units smaller or equal
+	       ;; to an hour since H:MM part takes care of it.
+	       (cl-remove-if-not
+	        (lambda (pair)
+		  (pcase pair
+		    (`(,(and unit (pred stringp)) . ,_)
+		     (> (org-duration--modifier unit canonical) 60))
+		    (_ nil)))
+	        duration-format))
+	      (min-modifier		;smallest modifier above hour
+	       (and truncated-format
+		    (apply #'min
+			   (mapcar (lambda (p)
+				     (org-duration--modifier (car p) canonical))
+				   truncated-format)))))
+         (if (or (null min-modifier) (< minutes min-modifier))
+	     ;; There is not unit above the hour or the smallest unit
+	     ;; above the hour is too large for the number of minutes we
+	     ;; need to represent.  Use H:MM or H:MM:SS syntax.
+	     (org-duration-from-minutes minutes mode canonical)
+	   ;; Represent minutes above hour using provided units and H:MM
+	   ;; or H:MM:SS below.
+	   (let* ((units-part (* min-modifier (/ (floor minutes) min-modifier)))
+		  (minutes-part (- minutes units-part))
+		  (compact (memq 'compact duration-format)))
+	     (concat
+	      (org-duration-from-minutes units-part truncated-format canonical)
+	      (and (not compact) " ")
+	      (org-duration-from-minutes minutes-part mode))))))
+      ;; Units format.
+      (duration-format
+       (let* ((fractional
+	       (let ((digits (cdr (assq 'special duration-format))))
+	         (and digits
+		      (or (wholenump digits)
+			  (error "Unknown formatting directive: %S" digits))
+		      (format "%%.%df" digits))))
+	      (selected-units
+	       (sort (cl-remove-if
+		      ;; Ignore special format cells and compact option.
+		      (lambda (pair)
+		        (pcase pair
+			  ((or `compact `(special . ,_)) t)
+			  (_ nil)))
+		      duration-format)
+		     (lambda (a b)
+		       (> (org-duration--modifier (car a) canonical)
+			  (org-duration--modifier (car b) canonical)))))
+	      (separator (if (memq 'compact duration-format) "" " ")))
+         (cond
+	  ;; Fractional duration: use first unit that is either required
+	  ;; or smaller than MINUTES.
+	  (fractional
+	   (let* ((unit (car
+		         (or (cl-find-if
+			      (lambda (pair)
+			        (pcase pair
+				  (`(,u . ,req?)
+				   (or req?
+				       (<= (org-duration--modifier u canonical)
+					  minutes)))))
+			      selected-units)
+			     ;; Fall back to smallest unit.
+			     (org-last selected-units))))
+		  (modifier (org-duration--modifier unit canonical)))
+	     (concat (format fractional (/ (float minutes) modifier)) unit)))
+	  ;; Otherwise build duration string according to available
+	  ;; units.
+	  ((org-string-nw-p
+	    (org-trim
+	     (mapconcat
+	      (lambda (units)
+	        (pcase-let* ((`(,unit . ,required?) units)
+			     (modifier (org-duration--modifier unit canonical)))
+		  (cond ((<= modifier minutes)
+		         (let ((value (floor minutes modifier)))
+			   (cl-decf minutes (* value modifier))
+			   (format "%s%d%s" separator value unit)))
+		        (required? (concat separator "0" unit))
+		        (t ""))))
+	      selected-units
+	      ""))))
+	  ;; No unit can properly represent MINUTES.  Use the smallest
+	  ;; one anyway.
+	  (t
+	   (pcase-let ((`((,unit . ,_)) (last selected-units)))
+	     (concat "0" unit)))))))))
 
 ;;;###autoload
 (defun org-duration-h:mm-only-p (times)
