@@ -1,4 +1,4 @@
-;;; test-org-bbdb.el --- tests for org-bbdb.el                        -*- lexical-binding: t; -*-
+;;; test-ol-bbdb.el --- tests for ol-bbdb.el                        -*- lexical-binding: t; -*-
 
 ;; Copyright (C) 2018, 2019  Marco Wahl
 
@@ -16,11 +16,11 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
-;; Test some of org-bbdb.el.
+;; Test some of ol-bbdb.el.
 
 ;;; Code:
 
@@ -31,6 +31,13 @@
   (should (equal '(9 22 2018) (org-bbdb-anniv-extract-date "2018-09-22")))
   (should (equal '(9 22 nil) (org-bbdb-anniv-extract-date "09-22"))))
 
+(ert-deftest test-org-bbdb-describe-link ()
+  (should (equal (org-bbdb-describe-link "bbdb:Richard Stallman" "RMS")
+                 "RMS"))
+  (should (equal (org-bbdb-describe-link "bbdb:Richard Stallman" "")
+                 "Richard Stallman"))
+  (should-not (org-bbdb-describe-link "https://example.com" "")))
+
 (provide 'test-ol-bbdb)
 
 ;;; test-org-bbdb.el ends here
