@@ -16,7 +16,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -24,9 +24,10 @@
 
 ;;; Code:
 
-(require 'org-test)
+(require 'org-test "../testing/org-test")
 (require 'org-attach)
-(eval-and-compile (require 'cl-lib))
+(eval-when-compile (require 'cl-lib))
+(require 'dired)
 
 (ert-deftest test-org-attach/dir ()
   "Test `org-attach-get' specifications."
@@ -116,7 +117,7 @@
 	     "* foo   :foo:"
 	   (split-window)
 	   (let ((org-buffer (current-buffer))
-		 (dired-buffer (dired temporary-file-directory)))
+		 (_dired-buffer (dired temporary-file-directory)))
 	     (cl-assert (eq 'dired-mode major-mode))
 	     (revert-buffer)
 	     (dired-goto-file a-filename)
@@ -130,10 +131,9 @@
 	   (search-forward "* foo")
 					; expectation.  tag ATTACH has been appended.
 	   (cl-reduce (lambda (x y) (or x y))
-		      (mapcar (lambda (x) (string-equal "ATTACH" x))
-			      (plist-get
-			       (plist-get
-				(org-element-at-point) 'headline) :tags))))
+		      (mapcar
+                       (lambda (x) (string-equal "ATTACH" x))
+                       (org-element-property :tags (org-element-at-point)))))
        (delete-file a-filename)))))
 
 (ert-deftest test-org-attach/dired-attach-to-next-best-subtree/2 ()
@@ -147,7 +147,7 @@
 	  "* foo"
 	  (split-window)
 	  (let ((org-buffer (current-buffer))
-		(dired-buffer (dired temporary-file-directory)))
+		(_dired-buffer (dired temporary-file-directory)))
 	    (cl-assert (eq 'dired-mode major-mode))
 	    (revert-buffer)
 	    (dired-goto-file a-filename)
