@@ -21,7 +21,7 @@
 
 ;;; Code:
 
-(eval-and-compile (require 'cl-lib))
+(eval-when-compile (require 'cl-lib))
 (require 'org-timer)
 
 (defmacro test-org-timer/with-temp-text (text &rest body)
